function [e,AF,AR] = swPemAFC_Modified_SC(Mic,Ls,AF,AR,UpdateFC,sel_alg)
%
%Update equation of time-domain LMS-based implementation of PEM-based
%adaptive feedback canceller
%
%INPUTS:
% * Mic = microphone sample
% * Ls  = loudspeaker sample
% * AF  = time-domain LMS-based feedback canceller and its properties
%          -AF.wTD: time-domain filter coefficients (dimensions: AF.N x 1)  
%          -AF.N  : time-domain filter length
%          -AF.mu : stepsize
%          -AF.p  : power in step-size normalization
%          -AF.lambda: weighing factor for power computation
%          -AF.TDLLs: time-delay line of loudspeaker samples
%          (dimensions: AF.N x 1)
%          -AF.TDLLswh: time-delay line of pre-whitened loudspeaker
%          samples (dimensions: AF.N x 1)
%
% * AR        = auto-regressive model and its properties
%                -AR.w           : coefficients of previous AR-model
%                -AR.N           : filter length AR model (Note: N=Nh+1)
%                -AR.framelength : framelength on which AR model is estimated
%                -AR.TDLMicdelay : time-delay line of microphone samples
%                                  (dimensions: AR.framelength+1 x 1)
%                -AR.TDLLsdelay  : time-delay line of loudspeaker samples
%                                  (dimensions: AR.framelength+1 x 1)
%                -AR.TDLMicwh    : time-delay line of pre-whitened
%                                  microphone signal
%                                  (dimensions: AR.N x 1)
%                -AR.TDLLswh     : time-delay line of pre-whitened
%                                  loudspeaker signal
%                                  (dimensions: AR.N x 1)
%                -AR.frame       : frame of AR.framelength error signals
%                                  on which AR model is computed  
%                -AR.frameindex
% * UpdateFC = boolean that indicates whether or not the feedback canceller should be updated 
%                    (1 = update feedback canceller; 0 = do not update feedback canceller)  
% * RemoveDC   = boolean that indicates whether or not the DC component of the estimated feedback path should be removed 
%                    (1 = remove DC of feedback canceller; 0 = do not remove DC of feedback canceller)  
%OUTPUTS:
% * e          = feedback-compensated signal 
% * AR         = updated AR-model and its properties
% * AF         = updated feedback canceller and its properties 
%
%
%
%Date: December, 2007  
%Copyright: (c) 2007 by Ann Spriet
%e-mail: ann.spriet@esat.kuleuven.be

% PEMSC-APA and swPEMSC
% Author: Linh Tran
% Date: March 2021 

%%%%%%%%%%%%%%%%%%%%%
AF.TDLLs = [Ls;AF.TDLLs(1:end-1,1)];
e     = Mic - AF.gTD'*AF.TDLLs;
hv = e;
e = 2*tanh(0.5*e);                      % limit the error signal
aaa = (abs(hv-e)<.15);

%Delay microphone and loudspeaker signal by framelength
[Micdelay,AR.TDLMicdelay] = DelaySample(Mic,AR.framelength,AR.TDLMicdelay); 
[Lsdelay,AR.TDLLsdelay] = DelaySample(Ls,AR.framelength,AR.TDLLsdelay);

% Filter microphone and loudspeaker signal with AR-model
[Micwh,AR.TDLMicwh] = FilterSample(Micdelay,AR.w,AR.TDLMicwh);
[Lswh,AR.TDLLswh]   = FilterSample(Lsdelay,AR.w,AR.TDLLswh);

%Update AR-model  
AR.frame=[e;AR.frame(1:AR.framelength-1)];
     
if and(AR.frameindex==AR.framelength-1,AR.N-1>0)
  R=zeros(AR.N,1);
  for j= 1:AR.N
     R(j,1) = (AR.frame'*[AR.frame(j:length(AR.frame)); zeros(j-1,1)])/AR.framelength;
  end
  [a,Ep] = levinson(R,AR.N-1);
  AR.w=a';
end

AR.frameindex = AR.frameindex+1;
      
if AR.frameindex == AR.framelength
  AR.frameindex = 0;
end

AF.TDLLswh = [Lswh;AF.TDLLswh(1:end-1,1)];
ep = Micwh - AF.gTD'*AF.TDLLswh;

if UpdateFC == 1    
       
    % APA alg. with pre-filters
    AF.TDLMicwh = [Micwh;AF.TDLMicwh(1:end-1,1)];   % size=(P,1);
    AF.TDLLswh_d = [Lswh;AF.TDLLswh_d(1:end-1,1)];  % size=(Lg_hat+P-1,1);      
    AF.Lswh_ap = AP_alg(AF.P,AF.N,AF.TDLLswh_d);    % size=(Lg_hat,P);    
    ewh_p = AF.TDLMicwh - AF.Lswh_ap'*AF.gTD;       % size=(P,1);
    
    % APA alg. without pre-filters
    AF.TDLMic = [Mic;AF.TDLMic(1:end-1,1)];         % size=(P,1);
    AF.TDLLs_d = [Ls;AF.TDLLs_d(1:end-1,1)];        % size=(Lg_hat+P-1,1);
    AF.Ls_ap = AP_alg(AF.P,AF.N,AF.TDLLs_d);        % size=(Lg_hat,P);
    e_ap = AF.TDLMic - AF.Ls_ap'*AF.gTD;            % size=(P,1);          
  
    if sel_alg == 0                
        % classical NLMS and APA (P=1 -> NLMS-l1; P=2 -> APA-l1)
        PEMSC_APA_term = AF.Lswh_ap*inv(AF.Lswh_ap'*AF.Lswh_ap+AF.delta*eye(AF.P))*ewh_p;
        AF.gTD = AF.gTD + AF.mu*PEMSC_APA_term;
        
    elseif sel_alg == 1      
        % swPEMSC: switch between PEMSC-NLMS and PEMSC-APA        
        PEMSC_NLMS_term = AF.TDLLswh .* conj(ep)/(norm(AF.TDLLswh)^2 + AF.delta);        
        PEMSC_APA_term = AF.Lswh_ap*inv(AF.Lswh_ap'*AF.Lswh_ap+AF.delta*eye(AF.P))*ewh_p;
        AF.gTD = AF.gTD + aaa*AF.mu1 * PEMSC_NLMS_term + (1-aaa)*AF.mu2 * PEMSC_APA_term;    
    
    else        
        % H-NLMS: switch NLMS, large mu (when the system is unstable) and PEMSC-NLMS, small mu (when the system is converged)       
        PEMSC_NLMS_term = AF.TDLLswh .* conj(ep)/(norm(AF.TDLLswh)^2 + AF.delta);
        NLMS_term = AF.TDLLs .* conj(e)/(norm(AF.TDLLs)^2 + AF.delta);
        AF.gTD = AF.gTD + aaa * AF.mu1 * PEMSC_NLMS_term + (1-aaa) * AF.mu2 * NLMS_term;
    end
    
% Remove DC
AF.gTD = AF.gTD - mean(AF.gTD);

end